@@ -13,7 +13,7 @@ Version 0.01
 
 =cut
 
-our $VERSION = '0.01';
+our $VERSION = '0.03';
 
 =head1 SYNOPSIS
 
@@ -288,6 +288,149 @@ B<Example:>
         top_p => 0.5
     );
 
+=head2 sample_top_k
+
+    my $token_id = $inference->sample_top_k(
+        \@logits,
+        temperature => 0.8,
+        top_k => 40
+    );
+
+Samples a token from logits using top-k sampling.
+
+B<Parameters:>
+
+=over 4
+
+=item * C<\@logits> - Array reference of logits from forward()
+
+=item * C<temperature> - Sampling temperature (default: 0.8)
+
+=item * C<top_k> - Number of top tokens to consider (default: 40)
+
+=back
+
+B<Returns:> A single token ID.
+
+B<Algorithm:>
+
+=over 4
+
+=item 1. Apply temperature scaling: logit / temperature
+
+=item 2. Convert to probabilities via softmax
+
+=item 3. Select top-k tokens by probability
+
+=item 4. Renormalize probabilities
+
+=item 5. Randomly sample from top-k tokens
+
+=back
+
+B<Example:>
+
+    my @logits = $inference->forward(\@tokens);
+    
+    # Sample from top 50 tokens
+    my $token = $inference->sample_top_k(\@logits,
+        temperature => 0.9,
+        top_k => 50
+    );
+
+=head2 generate
+
+    my @tokens = $inference->generate(
+        \@prompt_tokens,
+        max_tokens  => 100,
+        temperature => 0.8,
+        top_p       => 0.95,
+        top_k       => 40,
+        greedy      => 0,
+        eos_token   => 2,
+        callback    => sub { ... },
+    );
+
+Generates multiple tokens autoregressively from a prompt.
+
+B<Parameters:>
+
+=over 4
+
+=item * C<\@prompt_tokens> (required) - Array reference of prompt token IDs
+
+=item * C<max_tokens> - Maximum tokens to generate (default: 128)
+
+=item * C<temperature> - Sampling temperature (default: 0.8)
+
+=item * C<top_p> - Top-p (nucleus) sampling threshold (default: 0.95)
+
+=item * C<top_k> - Top-k sampling limit (default: 40). If < 1000, uses top_k; otherwise uses top_p
+
+=item * C<greedy> - If true, use greedy decoding (argmax) (default: 0)
+
+=item * C<eos_token> - Token ID to stop generation (default: from model, typically 2)
+
+=item * C<callback> - Optional subroutine called for each generated token
+
+=back
+
+B<Returns:> A list of generated token IDs (not including the prompt).
+
+B<Callback:>
+
+The callback receives (token_id, count) and should return true to stop generation:
+
+    callback => sub {
+        my ($token, $count) = @_;
+        print $tokenizer->decode([$token]);
+        return 0;  # Continue (return 1 to stop)
+    }
+
+B<Stopping Conditions:>
+
+Generation stops when:
+
+=over 4
+
+=item * max_tokens is reached
+
+=item * EOS token is generated
+
+=item * Callback returns true
+
+=back
+
+B<Example:>
+
+    use Lugh;
+    
+    my $model = Lugh::Model->new(model => 'model.gguf');
+    my $tokenizer = Lugh::Tokenizer->new(model => $model);
+    my $inference = Lugh::Inference->new(model => $model);
+    
+    my @prompt = $tokenizer->encode("Once upon a time");
+    
+    # Greedy generation
+    my @tokens = $inference->generate(\@prompt,
+        max_tokens => 50,
+        greedy     => 1,
+    );
+    print $tokenizer->decode(\@tokens);
+    
+    # Creative generation with streaming
+    @tokens = $inference->generate(\@prompt,
+        max_tokens  => 100,
+        temperature => 1.0,
+        top_p       => 0.95,
+        callback    => sub {
+            my ($tok, $n) = @_;
+            print $tokenizer->decode([$tok]);
+            STDOUT->flush();
+            return 0;
+        },
+    );
+
 =head1 ATTENTION MECHANISM
 
 =head2 Scaled Dot-Product Attention
@@ -382,37 +525,56 @@ Typical dimensions:
 
 =head1 GENERATION LOOP
 
-Complete generation example:
+The C<generate()> method handles the complete generation loop internally.
+For simple use cases:
 
-    sub generate {
-        my ($model, $tokenizer, $inference, $prompt, $max_tokens) = @_;
-        
-        my @tokens = $tokenizer->encode($prompt);
-        my @generated;
-        
-        for (1..$max_tokens) {
-            my @logits = $inference->forward(\@tokens);
-            
-            my $next = $inference->sample_top_p(\@logits,
-                temperature => 0.8,
-                top_p => 0.9
-            );
-            
-            last if $next == $tokenizer->eos_id;
-            
-            push @tokens, $next;
-            push @generated, $next;
-            
-            # Stream output
-            print $tokenizer->decode([$next]);
-            STDOUT->flush();
-        }
-        
-        return $tokenizer->decode(\@generated);
-    }
+    use Lugh;
     
-    my $response = generate($model, $tokenizer, $inference,
-        "Once upon a time", 100);
+    my $model = Lugh::Model->new(model => 'model.gguf');
+    my $tokenizer = Lugh::Tokenizer->new(model => $model);
+    my $inference = Lugh::Inference->new(model => $model);
+    
+    my @prompt = $tokenizer->encode("Once upon a time");
+    my @generated = $inference->generate(\@prompt,
+        max_tokens  => 100,
+        temperature => 0.8,
+        top_p       => 0.95,
+    );
+    print $tokenizer->decode(\@generated);
+
+For streaming output:
+
+    my @generated = $inference->generate(\@prompt,
+        max_tokens  => 100,
+        temperature => 0.8,
+        callback    => sub {
+            my ($token, $count) = @_;
+            print $tokenizer->decode([$token]);
+            STDOUT->flush();
+            return 0;  # Continue
+        },
+    );
+
+For manual control (building your own loop):
+
+    my @tokens = $tokenizer->encode($prompt);
+    my @generated;
+    
+    for (1..$max_tokens) {
+        my @logits = $inference->forward(\@tokens);
+        my $next = $inference->sample_top_p(\@logits,
+            temperature => 0.8,
+            top_p => 0.9
+        );
+        
+        last if $next == $tokenizer->eos_id;
+        
+        push @tokens, $next;
+        push @generated, $next;
+        
+        print $tokenizer->decode([$next]);
+        STDOUT->flush();
+    }
 
 =head1 PERFORMANCE
 
